// stratiGrapher: a tool to draw stratigraphic section in PostScript
// We are using libps library /n/sources/contrib/pac/sys/src/lib/libps.tbz
// Usage: sg < in (result is in draw.ps)

#include <u.h>
#include <libc.h>
#include <stdio.h>
#include <psgen.h>
#include "sg.h"

/* 
To Do:

more patterns and spotcolors
20*20 pt Bezier hatches
more definitions of rocks with proper color and hatch
wavy surfaces
copy upper surface to next bed's lower surface
handshake

Bugs:

bad pixels on right border of pattern, when viewed at 400%
*/

float linewidth;
int pspatterns[HATCHES];

void main(void) {
	Spotcolor spotcolors[NUMCOLORS], *colorptr;
	Rocktype rocktypes[NUMROCKS], *rockptr;
	FILE *colortab, *rocktab;
	PSDoc *ps;
	float x, y, thickness;
	int i, hardness,  rock, patt, invscale;

	// initialize
	ps = PS_new();



	if (0 > PS_open_file(ps, "draw.ps")) {
		printf("Cannot open PostScript file\n");
		exits("err open PS file");
	}
	linewidth = LINEWIDTH;
	invscale = INVSCALE;

	// read-in the spot color table
	colortab=fopen("spotcolors.in", "r");
	for(i=0; i<NUMCOLORS; i++) {
		if(fscanf(colortab,"%d %s %s %f %f %f %f", &spotcolors[i].id, spotcolors[i].name, spotcolors[i].colorspace, &spotcolors[i].c1, &spotcolors[i].c2, &spotcolors[i].c3, &spotcolors[i].c4)!=7) exits("err color file");
	}
	fclose(colortab);

	//make spot colors
	for(i=0; i<NUMCOLORS; i++) {
		PS_setcolor(ps, "fill", spotcolors[i].colorspace, spotcolors[i].c1, spotcolors[i].c2, spotcolors[i].c3, spotcolors[i].c4);
		spotcolors[i].id = PS_makespotcolor(ps, spotcolors[i].name, 0);
	}



	// read-in the rock properties table
	rocktab=fopen("rocks.in", "r");
	for(i=0; i<NUMROCKS; i++) {
		if(fscanf(rocktab,"%d %s %d %d %d ", &rocktypes[i].id, rocktypes[i].name, &rocktypes[i].fill, &rocktypes[i].hatch, &rocktypes[i].hardness)!=5) exits("err rocktable");
	}
	fclose(rocktab);

	// create rock patterns
	colorptr=spotcolors;
	rockptr=rocktypes;
	create_patterns(ps, pspatterns, colorptr, rockptr);

	// generate page
	PS_begin_page(ps, 596, 842);

	x = LEFT_BORDER;
	y = BOTTOM_BORDER;

	// read data line by line
	while(scanf("%f %d", &thickness, &rock)==2) {
		// recalc thicknes to points
		thickness *= C5/invscale;

		// get hardness, color and pattern
		hardness = rocktypes[rock].hardness;
		patt = pspatterns[rock];
		create_bed(ps, x, y, thickness, hardness, patt);
		y += thickness;
	};

	// draw thick line along the left side
	PS_setlinewidth(ps, 2*linewidth);
	PS_setcolor(ps, "stroke", "gray", 0.0, 0.0, 0.0, 0.0);
	PS_moveto(ps, LEFT_BORDER, BOTTOM_BORDER - linewidth/2);
	PS_lineto(ps, LEFT_BORDER, y + linewidth/2);
	PS_stroke(ps);

	// close all and exit
	PS_end_page(ps);
	PS_close(ps);
	PS_delete(ps);
	PS_shutdown();
	exits(0);

}
